@@ -370,9 +370,7 @@ class API(object):
 
         if self.debug:
             data['exception'] = '%s: %s' % (exc.__name__, inst)
-            data['traceback'] = '\n'.join(
-                traceback.format_exception(*exc_info)
-            )
+            data['traceback'] = traceback.format_exception(*exc_info)
 
         data = self.hook_500(data, request, exc_info)
 
@@ -547,7 +545,7 @@ class API(object):
             yield view
 
         for api in self.included:
-            for view in api.registered_views:
+            for view, url in api.registry:
                 yield view
 
     def resolve(self, paths):
