@@ -1,3 +1,4 @@
+from . import util
 from .query import DjangoQuery
 from django.core.urlresolvers import reverse
 from django.db import models
@@ -25,19 +26,40 @@ class DjangoModelResource(object):
 
     Options:
 
+        allowed_methods list of allowed HTTP methods for all endpoints
+        allowed_methods_endpoint
+                        list of allowed HTTP methods by endpoint
         excludes        list/tuple of field names to exclude from
                         serialize/deserialize
         fields          list/tuple of field names to serialize/deserialize
         use_location    instead of returning a serialized payload on POST/PUT,
                         use a Location header to point to where you may get
                         the representation (default: False)
+        version_field   field name to use as Optimistic Concurrency Control
+                        http://en.wikipedia.org/wiki/Optimistic_concurrency_control
+                        This field becomes required in all requests to update
+                        objects as it's tests for equality in the request
+                        compared to the database.  It's a way to ensure that
+                        if someone changes an object, it's changes are not
+                        accidently overwritten by another requester.  No server
+                        side locking required.
+                        caveat: disables PUT to list endpoint as there is
+                        no good way to handle this case (can be overridden by
+                        version_field_allow_put_list = True if you want to
+                        implement your own)
+        version_field_allow_put_list
+                        see version_field caveat
 
     '''
     Query = DjangoQuery
+    allowed_methods = None
+    allowed_methods_endpoint = None
     excludes = None
     fields = None
     model = None
     use_location = False
+    version_field = None
+    version_field_allow_put_list = None
 
     def __init__(self):
         self.fields = tuple([
@@ -60,6 +82,68 @@ class DjangoModelResource(object):
         self.loaded = False
         self.related_lookup_cache = {}
 
+        if self.allowed_methods:
+            self.allowed_methods = tuple([
+                method.lower()
+                for method in self.allowed_methods
+            ])
+
+        if self.allowed_methods_endpoint:
+            self.allowed_methods_endpoint = {
+                endpoint: tuple([
+                    method.lower()
+                    for method in methods
+                ])
+                for endpoint, methods in
+                    self.allowed_methods_endpoint.iteritems()
+                if methods
+            }
+
+        if self.version_field \
+           and self.version_field_allow_put_list is None:
+            self.version_field_allow_put_list = False
+
+    def allowed(self, endpoint, map):
+        '''allowed('list', {'get': self.get, ...})
+
+        Filter the given route method map based on specified allowed methods.
+        '''
+        tocheck = [
+            self.allowed_methods,
+            self.allowed_methods_endpoint.get(endpoint)
+                if self.allowed_methods_endpoint else None,
+        ]
+        newmap = {}
+        for method, view in map.iteritems():
+            allowed = True
+            for methods in tocheck:
+                if methods and method.lower() not in methods:
+                    allowed = False
+                    break
+
+            if allowed:
+                newmap[method] = view
+
+        return newmap
+
+    def clean(self, request, data, list=False):
+        '''clean(request, data) -> clean
+
+        Clean data from user environment suitable for applying to an instance
+        or creating an instance.
+        '''
+        expected = self.expected
+        if list:
+            expected = [expected]
+
+        try:
+            return self.api.expect(expected, data, strict_dict=False)
+        except self.api.ValueIssue as exc:
+            self.api.abort(self.api.serialize(request, 422, {
+                'error': 'Supplied data was invalid',
+                'invalid': exc.errors,
+            }))
+
     def clean_id(self, request, id):
         '''clean_id(request, id) -> clean_id
 
@@ -92,59 +176,68 @@ class DjangoModelResource(object):
                 'invalid': exc.errors,
             }))
 
+    def check(self, request, instance, clean, abortable=True):
+        '''check(request, instance, clean)
+
+        Aborts if any checks given an instance and the clean()ed data fail.
+
+        This is where version_field is used to test if the database and request
+        are in sync.
+
+        If abortable is False, the response code and error detail that would
+        have been serialized with the response is returned.
+
+            code, detail = \\
+                self.check(request, instance, clean, abortable=False)
+
+            if code:
+                if detail:
+                    self.api.abort(self.api.serialize(request, code, detail))
+
+                else:
+                    self.api.abort(request, code)
+
+        '''
+        # effectively use "goto" conventions, not really looping here
+        code = None
+        detail = None
+        while True:
+            if self.version_field and instance:
+                if self.version_field not in clean:
+                    code = 400
+                    detail = {
+                        'error': 'The "%s" field must be specified' \
+                        % self.version_field
+                    }
+                    break
+
+                version_db = getattr(instance, self.version_field)
+                version_clean = clean[self.version_field]
+                if version_db != version_clean:
+                    code = 409
+                    break
+
+            break
+
+        if abortable and code:
+            if detail:
+                self.api.abort(self.api.serialize(request, code, detail))
+            else:
+                self.api.abort(request, code)
+
+        return code, detail
+
     def deserialize(self, request, data, instance=None):
         '''deserialize(request, data, instance=instance) -> instance
 
-        Deserialize a request payload (`data`) into an instance (either given
-        instance or will construct a new one).
-
-        No further saving is required after this has been called.
+        Convenience method for clean(), check(), and update().
         '''
         if instance is None:
             instance = self.model()
 
-        try:
-            clean = self.api.expect(self.expected, data, strict_dict=False)
-        except self.api.ValueIssue as exc:
-            self.api.abort(self.api.serialize(request, 415, {
-                'error': 'Supplied data was invalid',
-                'invalid': exc.errors,
-            }))
-
-        manytomany = {}
-        for field in self.fields:
-            if field.name not in clean:
-                # since its not strictly enforcing extra/missing keys, if
-                # the client didn't supply the field, stick with model defaults
-                continue
-
-            method = getattr(self, 'deserialize_%s' % field.name, None)
-            if method:
-                setattr(instance, field.name, method(instance, field, data))
-
-            else:
-                if isinstance(field, models.ManyToManyField):
-                    resource = self.related_lookup(field)
-                    manytomany[field] = resource.fetch(*clean[field.name])
-
-                elif isinstance(field, models.ForeignKey):
-                    setattr(instance, field.name + '_id', clean[field.name])
-
-                else:
-                    setattr(instance, field.name, clean[field.name])
-
-        instance.save()
-        for field, instances in manytomany.iteritems():
-            manager = getattr(instance, field.name)
-            manager.clear()
-            manager.add(*instances)
-
-            if hasattr(instance, '_prefetched_objects_cache') \
-               and field.name in instance._prefetched_objects_cache:
-                instance._prefetched_objects_cache[field.name] = \
-                    instances
-
-        return instance
+        clean = self.clean(request, data)
+        self.check(request, instance, clean)
+        return self.update(instance, clean)
 
     def endpoint(self, name):
         '''endpoint('list') -> 'api_v1_resource_list'
@@ -288,7 +381,7 @@ class DjangoModelResource(object):
 
         Abort if the request is not authorized for the instance.
         '''
-        if not request.auth.is_authorized_obj(instance):
+        if not request.auth.is_authorized_object(instance):
             self.api.abort(self.api.challenge(request, request.auth))
 
     def load(self):
@@ -401,7 +494,7 @@ class DjangoModelResource(object):
         return a URI representing the path to the resource.
         '''
         resource = self.related_lookup(field)
-        if resource:
+        if resource and id is not None:
             return resource.reverse_single(id)
         else:
             return id
@@ -440,6 +533,52 @@ class DjangoModelResource(object):
         '''
         return self.reverse('single', args=(id,))
 
+    @property
+    def routes(self):
+        '''
+        Return a map of HTTP Method routes under an endpoint.
+
+        ie:
+            prefix = r'^%s/' % self.name
+            return (
+                ('name_of_url', prefix + r'endpoint/$', {
+                    'method': self.view,
+                    ...
+                }),
+                ...
+            )
+
+        '''
+        prefix = r'^%s/' % self.name
+        list_methods = {
+            'delete': self.delete_list,
+            'get': self.get_list,
+            'post': self.post_list,
+            'put': self.put_list,
+        }
+        if self.version_field and not self.version_field_allow_put_list:
+            del list_methods['put']
+
+        return (
+            (self.endpoint('list'), prefix + '$',
+                self.allowed('list', list_methods)
+            ),
+            (self.endpoint('single'), prefix + '(?P<id>[^/]+)/$',
+                self.allowed('single', {
+                    'delete': self.delete,
+                    'get': self.get,
+                    'put': self.put,
+                })
+            ),
+            (self.endpoint('set'), prefix + 'set/(?P<idset>[^/](?:[^/]|;)*)/$',
+                self.allowed('set', {
+                    'delete': self.delete_set,
+                    'get': self.get_set,
+                    'put': self.put_set,
+                })
+            ),
+        )
+
     def serialize(self, instance):
         '''serialize(instance) -> {...}
 
@@ -491,41 +630,47 @@ class DjangoModelResource(object):
 
         return data
 
-    @property
-    def routes(self):
-        '''
-        Return a map of HTTP Method routes under an endpoint.
+    def update(self, instance, clean):
+        '''update(instance, clean) -> instance
 
-        ie:
-            prefix = r'^%s/' % self.name
-            return (
-                ('name_of_url', prefix + r'endpoint/$', {
-                    'method': self.view,
-                    ...
-                }),
-                ...
-            )
+        Update an instance with clean()ed data specified in the request.
 
+        No further saving is required after this has been called.
         '''
-        prefix = r'^%s/' % self.name
-        return (
-            (self.endpoint('list'), prefix + '$', {
-                'delete': self.delete_list,
-                'get': self.get_list,
-                'post': self.post_list,
-                'put': self.put_list,
-            }),
-            (self.endpoint('single'), prefix + '(?P<id>[^/]+)/$', {
-                'delete': self.delete,
-                'get': self.get,
-                'put': self.put,
-            }),
-            (self.endpoint('set'), prefix + 'set/(?P<idset>[^/](?:[^/]|;)*)/$', {
-                'delete': self.delete_set,
-                'get': self.get_set,
-                'put': self.put_set,
-            }),
-        )
+        manytomany = {}
+        for field in self.fields:
+            if field.name not in clean:
+                # since its not strictly enforcing extra/missing keys, if
+                # the client didn't supply the field, stick with model defaults
+                continue
+
+            method = getattr(self, 'deserialize_%s' % field.name, None)
+            if method:
+                setattr(instance, field.name, method(instance, field, clean))
+
+            else:
+                if isinstance(field, models.ManyToManyField):
+                    resource = self.related_lookup(field)
+                    manytomany[field] = resource.fetch(*clean[field.name])
+
+                elif isinstance(field, models.ForeignKey):
+                    setattr(instance, field.name + '_id', clean[field.name])
+
+                else:
+                    setattr(instance, field.name, clean[field.name])
+
+        instance.save()
+        for field, instances in manytomany.iteritems():
+            manager = getattr(instance, field.name)
+            manager.clear()
+            manager.add(*instances)
+
+            if hasattr(instance, '_prefetched_objects_cache') \
+               and field.name in instance._prefetched_objects_cache:
+                instance._prefetched_objects_cache[field.name] = \
+                    instances
+
+        return instance
 
     @property
     def urls(self):
@@ -728,18 +873,9 @@ class DjangoModelResource(object):
 
         with atomic():
             instances = self.fetch_set(request, idset)
+            clean = self.clean(request, data, list=True)
 
-            try:
-                clean = self.api.expect([self.expected], data, strict_dict=False)
-            except self.api.ValueIssue as exc:
-                return self.api.serialize(request, 415, {
-                    'error': 'Data format was invalid',
-                    'invalid': exc.errors,
-                })
-
-            try:
-                ids = [self.clean_id(request, obj[self.pk_name]) for obj in clean]
-            except KeyError:
+            if not all(self.pk_name in obj for obj in clean):
                 return self.api.serialize(request, 400, {
                     'error': 'You must supply the primary key with each object'
                 })
@@ -748,8 +884,58 @@ class DjangoModelResource(object):
                 instance.pk: instance
                 for instance in instances
             }
+
+            code_and_details = []
             for obj in clean:
-                id = self.clean_id(request, obj[self.pk_name])
+                id = obj[self.pk_name]
+                instance = instance_lookup[id]
+                code, detail = \
+                    self.check(request, instance, obj, abortable=False)
+                if code:
+                    code_and_details.append((code, id, detail))
+
+            if code_and_details:
+                codes = list(set([
+                    code for code, id, detail in code_and_details
+                ]))
+                has_details = any(
+                    detail for code, id, detail in code_and_details
+                )
+                if len(codes) == 1:
+                    if has_details:
+                        self.api.abort(self.api.serialize(request, codes[0], {
+                            'invalid': {
+                                id: detail
+                                for code, id, detail in code_and_details
+                            }
+                        }))
+
+                    else:
+                        self.api.abort(request, codes[0])
+                else:
+                    # TODO really not thrilled with this... how is the client
+                    # supposed to handle this?
+                    # TODO figure out all 4xx responses that can come out of the
+                    # API and figure out a standardized way to handle the
+                    # response
+                    # at this point, at least they get _something_, but their
+                    # only reasonable course of action is to bail out completely
+                    # start documenting API 4xx errors page? and how to get
+                    # each one?
+                    # document them on a per method endpoint basis... seems like
+                    # the __doc__ on the method makes a lot of sense...
+                    self.api.abort(self.api.serialize(request, 400, {
+                        'error': 'Mixed error codes',
+                        'invalid': {
+                            id: util.defaults({}, detail or {}, {
+                                'code': code,
+                            })
+                            for code, id, detail in code_and_details
+                        }
+                    }))
+
+            for obj in clean:
+                id = obj[self.pk_name]
                 instance = instance_lookup[id]
                 self.deserialize(request, obj, instance)
 
