@@ -12,7 +12,7 @@ import json
 import pytest
 
 class Authz(AuthzBase):
-    def is_authorized_obj(self, request, authned, obj):
+    def is_authorized_object(self, request, authned, obj):
         return obj.__class__ is not ModelUnauthorized
 
 class Auth(AuthBase):
@@ -32,12 +32,18 @@ class ModelForeign(models.Model):
 class ModelForeign2(models.Model):
     char_indexed = models.CharField(max_length=20, db_index=True)
 
+class ModelForeign3(models.Model):
+    char_indexed = models.CharField(max_length=20, db_index=True)
+
 class ModelForeignNoResource(models.Model):
     name = models.CharField(max_length=20, db_index=True)
 
 class ModelForeignNoResourceForeign(models.Model):
     foreign = models.ForeignKey(ModelForeignNoResource)
 
+class ModelHasForeign3(models.Model):
+    foreign = models.ForeignKey(ModelForeign3)
+
 class ModelMany(models.Model):
     char_indexed = models.CharField(max_length=20, db_index=True)
 
@@ -52,9 +58,20 @@ class ModelUnauthorized(models.Model):
 
 class Model(models.Model):
     char_indexed = models.CharField(max_length=20, db_index=True)
-    foreign = models.ForeignKey(ModelForeign)
+    foreign = models.ForeignKey(ModelForeign, null=True)
     manytomany = models.ManyToManyField(ModelMany)
 
+class ModelAllowed(models.Model):
+    name = models.CharField(max_length=20)
+
+class ModelVersioned(models.Model):
+    name = models.CharField(max_length=20)
+    version = models.PositiveIntegerField(default=0)
+
+    def save(self, *args, **kwargs):
+        self.version += 1
+        return super(ModelVersioned, self).save(*args, **kwargs)
+
 @pytest.mark.django_db
 class ResourceTest(TestCaseNoDB):
     def _pre_setup(self):
@@ -75,20 +92,30 @@ class ResourceTest(TestCaseNoDB):
             model = ModelForeign2
             use_location = True
 
+        @api
+        class ModelForeign3Resource(DjangoModelResource):
+            model = ModelForeign3
+
         @api
         class ModelForeignNoResourceForeignResource(DjangoModelResource):
             model = ModelForeignNoResourceForeign
             name = 'modelforeignnoresourceforeign'
 
+        @api
+        class ModelHasForeign3Resource(DjangoModelResource):
+            model = ModelHasForeign3
+
+            def serialize_foreign(self, instance, field):
+                resource = self.related_lookup(field)
+                return resource.serialize(getattr(instance, field.name))
+
         @api
         class ModelManyResource(DjangoModelResource):
             model = ModelMany
-            name = 'modelmany'
 
         @api
         class ModelOtherResource(DjangoModelResource):
             model = ModelOther
-            name = 'modelother'
 
             def deserialize_seconds(self, instance, field, data):
                 return data['seconds'] / 10
@@ -99,12 +126,29 @@ class ResourceTest(TestCaseNoDB):
         @api
         class ModelUnauthorizedResource(DjangoModelResource):
             model = ModelUnauthorized
-            name = 'modelunauthorized'
 
         @api
         class ModelResource(DjangoModelResource):
             model = Model
-            name = 'model'
+
+        @api
+        class ModelAllowedResource(DjangoModelResource):
+            model = ModelAllowed
+            allowed_methods = ('get',)
+
+        @api
+        class ModelAllowed2Resource(DjangoModelResource):
+            model = ModelAllowed
+            allowed_methods_endpoint = {
+                'list': ('get',),
+                'single': ('get',),
+                'set': ('get',),
+            }
+
+        @api
+        class ModelVersionedResource(DjangoModelResource):
+            model = ModelVersioned
+            version_field = 'version'
 
         # only to satisfy coverage (calling load() twice)
         for resource in self.api.registered_resources:
@@ -124,6 +168,31 @@ class ResourceTest(TestCaseNoDB):
         ]:
             model.objects.all().delete()
 
+    def test_allowed(self):
+        ModelAllowed.objects.create(id=1, name='allowed')
+        ModelAllowed.objects.create(id=2, name='allowed2')
+
+        for code, method in (
+            (405, self.client.delete),
+            (200, self.client.get),
+            (405, self.client.post),
+            (405, self.client.put),
+        ):
+            for endpoint, args in (
+                ('list', ()),
+                ('single', (1,)),
+                ('set', ('1;2',)),
+            ):
+                response = method(
+                    self.api.reverse('modelallowed_' + endpoint, args=args)
+                )
+                assert response.status_code == code
+
+                response = method(
+                    self.api.reverse('modelallowed2_' + endpoint, args=args)
+                )
+                assert response.status_code == code
+
     def test_delete_list(self):
         ModelForeign.objects.create(id=1, char_indexed='value')
         ModelForeign.objects.create(id=2, char_indexed='value1')
@@ -313,6 +382,18 @@ class ResourceTest(TestCaseNoDB):
             'missing': [3],
         }
 
+    def test_get_set_trailing_separator(self):
+        response = self.client.get(
+            self.api.reverse('modelforeign_set', args=('1;2;',))
+        )
+
+        assert response.status_code == 400
+        assert response['Content-Type'] == 'application/json; charset=utf-8'
+        assert json.loads(response.content) == {
+            'error': 'Invalid ID for model ModelForeign',
+            'invalid': [u"invalid literal for int() with base 10: ''"],
+        }
+
     def test_get_single(self):
         ModelForeign.objects.create(id=1, char_indexed='value')
         response = self.client.get(
@@ -327,6 +408,23 @@ class ResourceTest(TestCaseNoDB):
             'resource_uri': '/api/v1/modelforeign/1/'
         }
 
+    def test_get_single_foreign_null(self):
+        Model.objects.create(id=1, foreign=None)
+
+        response = self.client.get(
+            self.api.reverse('model_single', args=(1,)),
+        )
+        assert response.status_code == 200
+        assert json.loads(response.content) == {
+            'id': 1,
+            'char_indexed': '',
+            'foreign': None,
+            'foreign_id': None,
+            'manytomany': [],
+            'manytomany_id': [],
+            'resource_uri': '/api/v1/model/1/',
+        }
+
     def test_get_single_missing(self):
         response = self.client.get(
             self.api.reverse('modelforeign_single', args=(1,))
@@ -643,10 +741,10 @@ class ResourceTest(TestCaseNoDB):
             content_type='application/json'
         )
 
-        assert response.status_code == 415
+        assert response.status_code == 422
         assert response['Content-Type'] == 'application/json; charset=utf-8'
         assert json.loads(response.content) == {
-            'error': 'Data format was invalid',
+            'error': 'Supplied data was invalid',
             'invalid': ["Unexpected type, expected <type 'list'>: <type 'dict'>"],
         }
 
@@ -770,7 +868,7 @@ class ResourceTest(TestCaseNoDB):
             }),
             content_type='application/json'
         )
-        assert response.status_code == 415
+        assert response.status_code == 422
         assert response['Content-Type'] == 'application/json; charset=utf-8'
         assert json.loads(response.content) == {
             'error': 'Supplied data was invalid',
@@ -850,7 +948,7 @@ class ResourceTest(TestCaseNoDB):
             content_type='application/json'
         )
 
-        assert response.status_code == 415
+        assert response.status_code == 422
         assert response['Content-Type'] == 'application/json; charset=utf-8'
         assert json.loads(response.content) == {
             'error': 'Supplied data was invalid',
@@ -895,6 +993,158 @@ class ResourceTest(TestCaseNoDB):
             'resource_uri': '/api/v1/model/1/',
         }
 
+    def test_serialize_foreign3(self):
+        ModelForeign3.objects.create(id=1, char_indexed='value')
+        ModelHasForeign3.objects.create(id=1, foreign_id=1)
+
+        response = self.client.get(
+            self.api.reverse('modelhasforeign3_single', args=(1,))
+        )
+        assert json.loads(response.content) == {
+            'id': 1,
+            'foreign': {
+                'id': 1,
+                'char_indexed': 'value',
+                'resource_uri': '/api/v1/modelforeign3/1/',
+            },
+            'resource_uri': '/api/v1/modelhasforeign3/1/',
+        }
+
+    def test_version_field(self):
+        ModelVersioned.objects.create(id=1, name='initial')
+
+        resource_uri = self.api.reverse('modelversioned_single', args=(1,))
+        response = self.client.put(
+            resource_uri,
+            json.dumps({
+                'name': 'first',
+                'version': 1,
+            }),
+            content_type='application/json'
+        )
+        assert response.status_code == 200
+        assert json.loads(response.content) == {
+            'id': 1,
+            'name': 'first',
+            'version': 2,
+            'resource_uri': resource_uri,
+        }
+
+        response = self.client.put(
+            resource_uri,
+            json.dumps({
+                'name': 'invalid',
+                'version': 1,
+            }),
+            content_type='application/json'
+        )
+        assert response.status_code == 409
+        assert response.content == ''
+
+        response = self.client.put(
+            resource_uri,
+            json.dumps({
+                'name': 'second',
+                'version': 2,
+            }),
+            content_type='application/json'
+        )
+        assert response.status_code == 200
+        assert json.loads(response.content) == {
+            'id': 1,
+            'name': 'second',
+            'version': 3,
+            'resource_uri': resource_uri,
+        }
+
+    def test_version_field_missing_version_field(self):
+        ModelVersioned.objects.create(id=1, name='initial')
+
+        resource_uri = self.api.reverse('modelversioned_single', args=(1,))
+        response = self.client.put(
+            resource_uri,
+            json.dumps({
+                'name': 'first',
+            }),
+            content_type='application/json'
+        )
+        assert response.status_code == 400
+        assert json.loads(response.content) == {
+            'error': 'The "version" field must be specified',
+        }
+
+    def test_version_field_put_set(self):
+        ModelVersioned.objects.create(id=1, name='initial')
+        ModelVersioned.objects.create(id=2, name='initial')
+
+        response = self.client.put(
+            self.api.reverse('modelversioned_set', args=('1;2',)),
+            json.dumps([
+                {
+                    'id': 1,
+                    'name': 'invalid',
+                },
+                {
+                    'id': 2,
+                    'name': 'invalid',
+                },
+            ]),
+            content_type='application/json'
+        )
+        assert response.status_code == 400
+        assert json.loads(response.content) == {
+            'invalid': {
+                '1': {'error': 'The "version" field must be specified'},
+                '2': {'error': 'The "version" field must be specified'},
+            }
+        }
+
+        response = self.client.put(
+            self.api.reverse('modelversioned_set', args=('1;2',)),
+            json.dumps([
+                {
+                    'id': 1,
+                    'name': 'invalid',
+                    'version': 2,
+                },
+                {
+                    'id': 2,
+                    'name': 'invalid',
+                    'version': 2,
+                },
+            ]),
+            content_type='application/json'
+        )
+        assert response.status_code == 409
+        assert response.content == ''
+
+        response = self.client.put(
+            self.api.reverse('modelversioned_set', args=('1;2',)),
+            json.dumps([
+                {
+                    'id': 1,
+                    'name': 'invalid',
+                },
+                {
+                    'id': 2,
+                    'name': 'invalid',
+                    'version': 2,
+                },
+            ]),
+            content_type='application/json'
+        )
+        assert response.status_code == 400
+        assert json.loads(response.content) == {
+            'error': 'Mixed error codes',
+            'invalid': {
+                '1': {
+                    'code': 400,
+                    'error': 'The "version" field must be specified'
+                },
+                '2': {'code': 409},
+            }
+        }
+
     @property
     def urls(self):
         from django.conf.urls import include, url
